package org.ibex.xt;
import org.ibex.js.*;
import org.ibex.util.*;
import org.ibex.io.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.thoughtworks.xstream.*;
import org.prevayler.*;
import org.prevayler.implementation.snapshot.*;

public class Servlet extends HttpServlet {

    private ServletScope servletscope = null;
    private String path;
    private Prevayler prevayler;
    private JS prevalent;
    private ServletContext cx = null;

    public void destroy() { try {
        synchronized(this.getClass()) {
            Prevayler privatePrevayler = prevayler;
            if (prevayler == null) return;
            prevayler = null;
            Prevalence.destroy(cx, prevayler);
        }
    } catch (Exception e) { e.printStackTrace(); } }

    public void init(ServletConfig sc) throws ServletException {
        cx = sc.getServletContext();
        prevayler = Prevalence.getPrevayler(cx);
        prevalent = (JS)prevayler.prevalentSystem();
    }

    public void doPost(HttpServletRequest request, HttpServletResponse response) throws IOException { doGet(request, response); }
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {
        servletscope = new ServletScope(request, response, cx);
        path = cx.getRealPath(((HttpServletRequest)request).getServletPath());
        Reader xmlreader = new InputStreamReader(new FileInputStream(path));
        new Template(servletscope, new JSScope(servletscope), xmlreader).wrap(null).toXML(response.getWriter());
    }

    public class ServletScope extends JSScope {
        HttpServletRequest request;
        HttpServletResponse response;
        ServletContext cx;
        public String getRealPath(String s) { return cx.getRealPath(s); }
        public ServletScope(ServletRequest request, ServletResponse response, ServletContext cx) {
            super(null);
            this.request = (HttpServletRequest)request;
            this.response = (HttpServletResponse)response;
            this.cx = cx;
        }
        private JS params = new JS() {
                public Object get(Object key) { return request.getParameter(JS.toString(key)); }
                public Enumeration keys() { return request.getParameterNames(); }
            };
        private JS cookies = new JS() {
                /*
                public Object get(Object key) { return request.getCookie(JS.toString(key)); }
                public Enumeration keys() { return request.getCookieNames(); }
                */
            };
        private JS sessionAttributes = new JS() {
                public Object get(Object key) { return request.getSession(true).getAttribute(JS.toString(key)); }
                public void put(Object key, Object val) {
                    if (val == null) request.getSession(true).removeAttribute(JS.toString(key));
                    else request.setAttribute(JS.toString(key), val); }
                public Enumeration keys() { return request.getSession(true).getAttributeNames(); }
            };
        private JS requestHeader = new JS() {
                public Object get(Object key) { return request.getHeader(JS.toString(key)); }
                public Enumeration keys() { return request.getHeaderNames(); }
            };
        private JS responseHeader = new JS() {
                public void put(Object key, Object val) { response.setHeader(JS.toString(key), JS.toString(val)); }
            };


        /** lets us put multi-level get/put/call keys all in the same method */
        private class Sub extends JS {
            Object key;
            Sub(Object key) { this.key = key; }
            public void put(Object key, Object val) throws JSExn {
                ServletScope.this.put(JS.toString(this.key) + "." + JS.toString(key), val); }
            public Object get(Object key) throws JSExn {
                return ServletScope.this.get(JS.toString(this.key) + "." + JS.toString(key)); }
            public Object call(Object a0, Object a1, Object a2, Object[] rest, int nargs) throws JSExn {
                return ServletScope.this.callMethod(this.key, a0, a1, a2, rest, nargs);
            }
            public Object callMethod(Object method, Object a0, Object a1, Object a2, Object[] rest, int nargs) throws JSExn {
                return ServletScope.this.callMethod(JS.toString(this.key) + "."
                                                         + JS.toString(method), a0, a1, a2, rest, nargs);
            }
        }
        private Sub getSub(String key) { return new Sub(key); }

        public Object callMethod(Object method, final Object a, final Object b, Object c, Object[] rest, int nargs) throws JSExn {
            //#switch(method)
            case "session.invalidate":    request.getSession(true).invalidate(); return null;
            case "context.list":
                String path = JS.toString(a);
                if (path.indexOf("..") != -1) throw new JSExn("cannot use .. in paths");
                File f = new File(cx.getRealPath("/") + File.separatorChar + path);
                if (!f.isDirectory()) return null;
                String[] contents = f.list();
                JSArray ret = new JSArray(contents.length);
                for(int i=0; i<contents.length; i++) ret.addElement(contents[i]);
                return ret;
            //#end
            return null;
        }
        public Object get(Object key) throws JSExn {
            //#switch(key)
            case "body":
            case "arg":                   return null;
            case "prevalent":             return prevayler.prevalentSystem();
            case "request":               return getSub("request");
            case "request.user":          return request.getRemoteUser();
            case "request.header":        return requestHeader;
            case "request.method":        return request.getMethod();
            case "request.remote":        return getSub("request.remote");
            case "request.remote.ip":     return request.getRemoteAddr();
            case "request.remote.host":   return request.getRemoteHost();
            case "request.ssl":           return request.isSecure();
            case "request.path":          return request.getPathInfo();
            case "response":              return getSub("response");
            case "response.header":       return responseHeader;
            case "session":               return getSub("session");
            case "session.attr":          return sessionAttributes;
            case "session.created":       return new JSDate(request.getSession(true).getCreationTime());
            case "session.accessed":      return new JSDate(request.getSession(true).getLastAccessedTime());
            case "session.invalidate":    return METHOD;
            case "page":                  return getSub("page");
            case "page.lastmodified":     return new JSDate(new File(path).lastModified());
            case "context":               return getSub("context");
            case "context.list":          return METHOD;
            case "params":                return params;
            case "cookie":                return cookies;
            //#end
            return null;
        }
        public void put(Object key, Object val) throws JSExn {
            try {
            //#switch(JS.toString(key))
            case "transaction":
                try { prevayler.execute(new Prevalence.JSTransaction((JS)val));
                } catch (Exception e) { e.printStackTrace(); throw new RuntimeException(e); }
            case "response.code":         response.setStatus(JS.toInt(val));
            case "response.redirect":     response.sendRedirect(JS.toString(val));
            case "response.contentType":  response.setContentType(JS.toString(val));
            //#end
            } catch (IOException e) {
                throw new JSExn(e);
            }
        }
    }

}
